@@ -8,4 +8,4 @@
 #  _________________________________________________________________________
 #
 
-from processmngr import *
+from pyutilib.subprocess.processmngr import *
