@@ -9,7 +9,7 @@
 
 __all__ = ['subprocess', 'SubprocessMngr', 'run_command', 'timer', 'signal_handler', 'run']
 
-import GlobalData
+from pyutilib.subprocess import GlobalData
 import time
 import signal
 import os
@@ -33,7 +33,7 @@ def kill_process(process, sig=signal.SIGTERM, verbose=False):
     """
     pid = process.pid
     if GlobalData.debug or verbose:
-        print "Killing process",pid,"with signal",sig
+        print("Killing process %d with signal %d" % (pid,sig))
     if subprocess.mswindows:
         if sys.version_info[0:2] < (2,5):
             os.system("taskkill /t /f /pid "+repr(pid))
@@ -48,7 +48,7 @@ def kill_process(process, sig=signal.SIGTERM, verbose=False):
         #
         pgid=os.getpgid(pid)
         if pgid == -1:
-            print "  ERROR: invalid pid",pid
+            print("  ERROR: invalid pid %d" % pid)
             sys.exit(1)
         os.killpg(pgid,signal.SIGTERM)
         #
@@ -71,18 +71,18 @@ GlobalData.signal_handler_busy=False
 #
 def verbose_signal_handler(signum, frame):
     c = frame.f_code
-    print '  Signal handler called from ', c.co_filename, c.co_name, frame.f_lineno
-    print "  Waiting...",
+    print('  Signal handler called from ', c.co_filename, c.co_name, frame.f_lineno)
+    print("  Waiting...",)
     signal_handler(signum, frame, True)
 
 def signal_handler(signum, frame, verbose=False):
     if GlobalData.signal_handler_busy:
-        print ""
-        print "  Signal handler is busy.  Aborting."
+        print("")
+        print("  Signal handler is busy.  Aborting.")
         sys.exit(-signum)
     if GlobalData.current_process is None:
-        print "  Signal",signum,"recieved, but no process queued"
-        print "  Exiting now"
+        print("  Signal",signum,"recieved, but no process queued")
+        print("  Exiting now")
         sys.exit(-signum)
     if GlobalData.current_process is not None and\
        GlobalData.current_process.pid is not None and\
@@ -90,7 +90,7 @@ def signal_handler(signum, frame, verbose=False):
         GlobalData.signal_handler_busy=True
         kill_process(GlobalData.current_process, signum)
         if verbose:
-            print "  Signaled process", GlobalData.current_process.pid,"with signal",signum
+            print("  Signaled process", GlobalData.current_process.pid,"with signal",signum)
         endtime = timer()+1.0
         while timer() < endtime:
             status = GlobalData.current_process.poll()
@@ -102,12 +102,12 @@ def signal_handler(signum, frame, verbose=False):
         if status is not None:
             GlobalData.signal_handler_busy=False
             if verbose:
-                print "Done."
-            raise OSError, "Interrupted by signal " + repr(signum)
+                print("Done.")
+            raise OSError("Interrupted by signal " + repr(signum))
         else:
-            raise OSError, "Problem terminating process" + repr(GlobalData.current_process.pid)
+            raise OSError("Problem terminating process" + repr(GlobalData.current_process.pid))
         GlobalData.current_process = None
-    raise OSError, "Interrupted by signal " + repr(signum)
+    raise OSError("Interrupted by signal " + repr(signum))
 
 
 #
@@ -116,37 +116,18 @@ def signal_handler(signum, frame, verbose=False):
 def _read_data(stream, ostr1, ostr2):
     while True:
         data = os.read(stream, 80)
+        if sys.stdout.encoding is None:
+            data = data.decode('utf-8')
+        else:
+            data = data.decode(sys.stdout.encoding)
         if data:
             ostr1.write(data)
-            ostr2.write(data)
+            #ostr2.write(data)
             ostr1.flush()
-            ostr2.flush()
+            #ostr2.flush()
         else:
             break
 
-#
-# A depricated read_data function
-#
-def X_read_data(stream, ostr1, ostr2):
-    li = [stream]
-    ctr = 0
-    while ctr < 3:
-        i,o,e = select.select(li,[],[], 0.0)
-        if i:
-            data = os.read(stream,2048)
-            if data:
-                ostr1.write(data)
-                ostr2.write(data)
-                ostr1.flush()
-                ostr2.flush()
-            else:
-                li = []
-            ctr = 0
-        else:
-            ctr += 1
-        if ctr:
-            time.sleep(1.0)
-
 #
 # Execute the command as a subprocess that we can send signals to.
 # After this is finished, we can get the output from this command from
@@ -175,7 +156,7 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdin=None, stdout=No
     if memmon:
         memmon = pyutilib.services.registered_executable("memmon")
         if memmon is None:
-            raise IOError, "Unable to find the 'memmon' executable"
+            raise IOError("Unable to find the 'memmon' executable")
         _cmd.insert(0, memmon.get_path())
     #
     # Setup valgrind
@@ -187,7 +168,7 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdin=None, stdout=No
         #
         valgrind_cmd = pyutilib.services.registered_executable("valgrind")
         if valgrind_cmd is None:
-            raise IOError, "Unable to find the 'valgrind' executable"
+            raise IOError("Unable to find the 'valgrind' executable")
         valgrind_cmd = [ valgrind_cmd.get_path() ]
         if valgrind_options is None:
             valgrind_cmd.extend(
@@ -244,7 +225,7 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdin=None, stdout=No
             signal.signal(signal.SIGTERM, signal_handler)
     rc = -1
     if debug:
-        print "Executing command %s" % (_cmd, )
+        print("Executing command %s" % (_cmd, ))
     try:
         GlobalData.signal_handler_busy=False
         if (tee is None) or (tee is False):
@@ -262,7 +243,6 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdin=None, stdout=No
             # as doing a normal 'print'
             #
             rpipe1, wpipe1 = os.pipe()
-            ##rpipe2, wpipe2 = os.pipe()
             #
             process = SubprocessMngr(_cmd, stdin=stdin, stdout=wpipe1, stderr=wpipe1, env=env, shell=shell)
             GlobalData.current_process = process.process
@@ -273,9 +253,6 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdin=None, stdout=No
             outt = Thread(target=_read_data, args=(rpipe1, sys.stdout, stdout_arg))
             outt.daemon = True
             outt.start()
-            ##errt = Thread(target=_read_data, args=(rpipe2, sys.stderr, stderr_arg))
-            ##errt.daemon = True
-            ##errt.start()
             #
             # Wait for process to finish
             #
@@ -285,18 +262,15 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdin=None, stdout=No
             # 'Closing' the PIPE to send EOF to the reader.
             #
             os.close(wpipe1)
-            ##os.close(wpipe2)
             #
             # Wait for readers to finish up with the data in the pipe.
             #
             outt.join()
-            ##errt.join()
             del outt
-            ##del errt
             os.close(rpipe1)
-            ##os.close(rpipe2)
 
-    except WindowsError, err:
+    except WindowsError:
+        err = sys.exc_info[1]
         raise ApplicationError(
             "Could not execute the command: '%s'\n\tError message: %s"
             % (' '.join(_cmd), err) )
@@ -305,7 +279,7 @@ def run_command(cmd, outfile=None, cwd=None, ostream=None, stdin=None, stdout=No
         # Ignore IOErrors, which are caused by interupts
         #
         pass
-    if ostream is None and outfile is not None:
+    if ostream is None or outfile is not None:
         stdout_arg.close()
     elif not tmpfile is None and not ignore_output:
         tmpfile.seek(0)
@@ -386,7 +360,6 @@ class SubprocessMngr(object):
         if False and self.process is not None:
             try:
                 if self.process.poll() is None:
-                    print "X",self.process.pid
                     self.kill()
             except OSError:
                 #
@@ -418,7 +391,7 @@ class SubprocessMngr(object):
             # Sleep every 1/10th of a second to avoid wasting CPU time
             #
             if timelimit <= 0:
-                raise ValueError, "'timeout' must be a positive number"
+                raise ValueError("'timeout' must be a positive number")
             endtime = timer()+timelimit
 
             # This might be dangerous: we *could* deadlock if the input
@@ -453,7 +426,7 @@ class SubprocessMngr(object):
                 if status is not None:
                     return status
                 else:
-                    raise OSError, "Could not kill process " + repr(self.process.pid)
+                    raise OSError("Could not kill process " + repr(self.process.pid))
 
     def stdout(self):
         return self.process.stdout
@@ -475,54 +448,54 @@ class SubprocessMngr(object):
 
 if __name__ == "__main__": #pragma:nocover
     #GlobalData.debug=True
-    print "Z"
+    print("Z")
     stime = timer()
     foo = run_command("./dummy", tee=True, timelimit=10)
-    print "A"
-    print "Ran for " + repr(timer()-stime) + " seconds"
-    print foo
+    print("A")
+    print("Ran for " + repr(timer()-stime) + " seconds")
+    print(foo)
     sys.exit(0)
 
     if not subprocess.mswindows:
         foo = SubprocessMngr("ls *py")
         foo.wait()
-        print ""
+        print("")
 
         foo = SubprocessMngr("ls *py", stdout=subprocess.PIPE)
         foo.wait()
         for line in foo.process.stdout:
-            print line,
-        print ""
+            print(line,)
+        print("")
         foo=None
 
         [rc,output] = run_command("ls *py")
-        print output
-        print ""
+        print(output)
+        print("")
 
         [rc,output] = run_command("ls *py", outfile="tmp")
         INPUT=open("tmp",'r')
         for line in INPUT:
-            print line,
+            print(line,)
         INPUT.close()
-        print ""
+        print("")
 
-        print "X"
+        print("X")
         [rc,output] = run_command("python -c \"while True: print '.'\"",
                                   timelimit=2)
-        print "Y"
+        print("Y")
         #[rc,output] = run_command("python -c \"while True: print '.'\"")
         [rc,output] = run_command("python -c \"while True: print '.'\"", verbose=False)
-        print "Y-end"
+        print("Y-end")
     else:
         foo = SubprocessMngr("cmd /C \"dir\"")
         foo.wait()
-        print ""
+        print("")
 
-    print "Z"
+    print("Z")
     stime = timer()
     foo = run_command("python -c \"while True: pass\"", timelimit=10)
-    print "A"
-    print "Ran for " + repr(timer()-stime) + " seconds"
+    print("A")
+    print("Ran for " + repr(timer()-stime) + " seconds")
 
 
 pyutilib.services.register_executable("valgrind")
